// TMGRealTimeAudioDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TMGSDK_For_Audio_ApiExample.h"
#include "TMGRealTimeAudioDlg.h"
#include "afxdialogex.h"
#include <sstream>
#include <algorithm>
#include "tmg_sdk.h"
#include "Utils.h"
#include "jsoncpp\json.h"
#include "av_error.h"



#define TMG_INIT_ROOMID "20210801"		//The audio room's id, in the same room id, OpenIDs can talk with each other.


// CTMGRealTimeAudioDlg dialog

IMPLEMENT_DYNAMIC(CTMGRealTimeAudioDlg, CDialogEx)

CTMGRealTimeAudioDlg::CTMGRealTimeAudioDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(IDD_DIALOG_RTAUDIO, pParent)
{

}

CTMGRealTimeAudioDlg::~CTMGRealTimeAudioDlg()
{
}

void CTMGRealTimeAudioDlg::AppendCallbackInfo2Monitor(const std::string& callbackINfo)
{
	if (mMonitorLog.GetLength() > 40960)
	{
		mMonitorLog = "";
	}
	if (!mMonitorLog.IsEmpty())
	{
		mMonitorLog.Append(L"\r\n");
	}
	mMonitorLog.Append(CUtils::UTF8ToWChar(callbackINfo).c_str());
	m_EditMonitor.SetWindowText(mMonitorLog);
	m_EditMonitor.LineScroll(100000);
}

void CTMGRealTimeAudioDlg::SetHostDialog(CTMGSDK_For_Audio_ApiExampleDlg *pHostDlg)
{
	m_pHostDlg = pHostDlg;
}

void CTMGRealTimeAudioDlg::HandleEnterRoomCallback(const char* data)
{
	std::string strText = u8"EnterRoom complete: ret=";
	strText.append(data);

	if (strText.find("\"result\":0") == strText.npos)
	{
		m_EnterRoomBtn.EnableWindow(true);
		
	} else {
		ITMGContextGetInstance()->GetAudioEffectCtrl()->InitVoiceChanger("./VoiceData");
		ITMGContextGetInstance()->GetAudioEffectCtrl()->FetchVoiceChangerList();
	}
	
	AppendCallbackInfo2Monitor(strText);
}

void CTMGRealTimeAudioDlg::HandleUserInfoUpdateCallback(const char* data)
{
	std::string strText = u8"UserUpdate complete: ret=";
	strText.append(data);
	AppendCallbackInfo2Monitor(strText);
}
void CTMGRealTimeAudioDlg::HandleDisconnectRoomCallback(const char* data)
{
	std::string strText = u8"UserUpdate complete: ret=";
	strText.append(data);
	AppendCallbackInfo2Monitor(strText);
	m_EnableMicCheckBox.SetCheck(false);
	m_EnableSpeakerCheckBox.SetCheck(false);
	m_EnableLoopbackCheckBox.SetCheck(false);

    ITMGContextGetInstance()->GetAudioEffectCtrl()->SetVoiceChangerName("");
	ShowAllParamViews(false);
}
void CTMGRealTimeAudioDlg::HandleExitRoomCallback(const char* data)
{
	std::string strText = u8"ExitRoom complete: ret=";
	strText.append(data);
	AppendCallbackInfo2Monitor(strText);
	m_EnableMicCheckBox.SetCheck(false);
	m_EnableSpeakerCheckBox.SetCheck(false);
	m_EnableLoopbackCheckBox.SetCheck(false);
	m_EnterRoomBtn.EnableWindow(true);

    ITMGContextGetInstance()->GetAudioEffectCtrl()->SetVoiceChangerName("");
	ShowAllParamViews(false);
}


void CTMGRealTimeAudioDlg::HandleRoomTypeChangeCallback(const char* data)
{
	std::string strText = u8"Room type changed: ret=";
	strText.append(data);
	AppendCallbackInfo2Monitor(strText);

	std::string strData = data;
	if (strData.find("\"result\":0") != std::string::npos)
	{
		if (strData.find("\"sub_event_type\":1") != std::string::npos || strData.find("\"sub_event_type\":3") != std::string::npos)
		{
			if (strData.find("\"new_room_type\":1") != std::string::npos)
			{
				m_ComBoxRoomType.SetCurSel(0);
			}else if (strData.find("\"new_room_type\":2") != std::string::npos)
			{
				m_ComBoxRoomType.SetCurSel(1);
			}
			else if (strData.find("\"new_room_type\":3") != std::string::npos)
			{
				m_ComBoxRoomType.SetCurSel(2);
			}
		}
	}
}

void CTMGRealTimeAudioDlg::HandleInited() {
	// fetch voiceList
	//ITMGContextGetInstance()->GetAudioEffectCtrl()->FetchVoiceChangerList();
}

void CTMGRealTimeAudioDlg::HandleFetchVoiceListCallback(const char* data) {
	int a = 1;
	int b = 2;
	int c = a + b;

	nsgme::av::Json::Reader reader;
	nsgme::av::Json::Value root;
	bool parseRet = reader.parse(data, root);

	if (!parseRet) {
		AppendCallbackInfo2Monitor(std::string("parse fetch voice list result Json error"));
		return;
	}
	int result = root["result"].asInt();
	if (result != 0) {
		AppendCallbackInfo2Monitor(std::string("fetch voice error"));
		return;
	}
	nsgme::av::Json::Value voice_list = root["voice_list"];

	m_ComBoxVoiceType.ResetContent();
	m_vVoiceList.push_back("None");
	m_ComBoxVoiceType.InsertString(0, L"None");
	int listSize = (int)voice_list.size();
	std::string voiceType;
	for (int i = 0; i < listSize; i++) {
		voiceType = voice_list[i].asString();
		m_vVoiceList.push_back(voiceType);
		m_ComBoxVoiceType.InsertString(i+1, CUtils::UTF8ToWChar(voiceType).c_str());
	}
	m_ComBoxVoiceType.SetCurSel(0);
}


void CTMGRealTimeAudioDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT_ROOMID, m_RoomId);
	DDX_Control(pDX, IDC_COMBO_ROOMTYPE, m_ComBoxRoomType);
	DDX_Control(pDX, IDC_COMBO_VOICETYPE, m_ComBoxVoiceType);
	DDX_Control(pDX, IDC_EDIT_AUDIO_MONITOR, m_EditMonitor);
	DDX_Control(pDX, IDC_CHECK_ENABLEMIC, m_EnableMicCheckBox);
	DDX_Control(pDX, IDC_CHECK_ENABLESPEAKER, m_EnableSpeakerCheckBox);
	DDX_Control(pDX, IDC_CHECK_ENABLELOOPBACK, m_EnableLoopbackCheckBox);
	DDX_Control(pDX, IDC_BUTTON_ENTERROOM, m_EnterRoomBtn);
	DDX_Control(pDX, IDC_BUTTON_RESETPARAM, m_ResetParamBtn);
	DDX_Control(pDX, IDC_STATIC_PARAM1, m_ParamNameView1);
	DDX_Control(pDX, IDC_STATIC_PARAM2, m_ParamNameView2);
	DDX_Control(pDX, IDC_STATIC_PARAM3, m_ParamNameView3);
	DDX_Control(pDX, IDC_STATIC_PARAM4, m_ParamNameView4);
	DDX_Control(pDX, IDC_STATIC_PARAM5, m_ParamNameView5);
	DDX_Control(pDX, IDC_STATIC_PARAM6, m_ParamNameView6);
	DDX_Control(pDX, IDC_STATIC_PARAM7, m_ParamNameView7);
	DDX_Control(pDX, IDC_STATIC_PARAM8, m_ParamNameView8);
	DDX_Control(pDX, IDC_STATIC_PARAM9, m_ParamNameView9);
	DDX_Control(pDX, IDC_STATIC_PARAM10, m_ParamNameView10);
	DDX_Control(pDX, IDC_STATIC_VALUE1, m_ParamValueView1);
	DDX_Control(pDX, IDC_STATIC_VALUE2, m_ParamValueView2);
	DDX_Control(pDX, IDC_STATIC_VALUE3, m_ParamValueView3);
	DDX_Control(pDX, IDC_STATIC_VALUE4, m_ParamValueView4);
	DDX_Control(pDX, IDC_STATIC_VALUE5, m_ParamValueView5);
	DDX_Control(pDX, IDC_STATIC_VALUE6, m_ParamValueView6);
	DDX_Control(pDX, IDC_STATIC_VALUE7, m_ParamValueView7);
	DDX_Control(pDX, IDC_STATIC_VALUE8, m_ParamValueView8);
	DDX_Control(pDX, IDC_STATIC_VALUE9, m_ParamValueView9);
	DDX_Control(pDX, IDC_STATIC_VALUE10, m_ParamValueView10);
	DDX_Control(pDX, IDC_SLIDER_PARAM1, m_ParamSliderView1);
	DDX_Control(pDX, IDC_SLIDER_PARAM2, m_ParamSliderView2);
	DDX_Control(pDX, IDC_SLIDER_PARAM3, m_ParamSliderView3);
	DDX_Control(pDX, IDC_SLIDER_PARAM4, m_ParamSliderView4);
	DDX_Control(pDX, IDC_SLIDER_PARAM5, m_ParamSliderView5);
	DDX_Control(pDX, IDC_SLIDER_PARAM6, m_ParamSliderView6);
	DDX_Control(pDX, IDC_SLIDER_PARAM7, m_ParamSliderView7);
	DDX_Control(pDX, IDC_SLIDER_PARAM8, m_ParamSliderView8);
	DDX_Control(pDX, IDC_SLIDER_PARAM9, m_ParamSliderView9);
	DDX_Control(pDX, IDC_SLIDER_PARAM10, m_ParamSliderView10);

}

void CTMGRealTimeAudioDlg::ShowAllParamViews(bool bShow) {
	for (size_t i = 0; i < COUNT_DEFAULT_PARAM_VIEW; i++) {
		m_ParamNameViews[i]->ShowWindow(bShow);
		m_ParamValueViews[i]->ShowWindow(bShow);
		m_ParamSliderViews[i]->ShowWindow(bShow);
	}
	m_ResetParamBtn.ShowWindow(bShow);
}

BOOL CTMGRealTimeAudioDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// init subViews
	m_RoomId.SetWindowText(_T(TMG_INIT_ROOMID));
	m_ComBoxRoomType.AddString(L"1_ROOM_TYPE_FLUENCY");
	m_ComBoxRoomType.AddString(L"2_ROOM_TYPE_STANDARD");
	m_ComBoxRoomType.AddString(L"3_ROOM_TYPE_HIGHQUALITY");
	m_ComBoxRoomType.SetCurSel(0);
	m_ComBoxVoiceType.AddString(L"None");
	m_ComBoxVoiceType.SetCurSel(0);



	m_ParamNameViews[0] = &m_ParamNameView1;
	m_ParamNameViews[1] = &m_ParamNameView2;
	m_ParamNameViews[2] = &m_ParamNameView3;
	m_ParamNameViews[3] = &m_ParamNameView4;
	m_ParamNameViews[4] = &m_ParamNameView5;
	m_ParamNameViews[5] = &m_ParamNameView6;
	m_ParamNameViews[6] = &m_ParamNameView7;
	m_ParamNameViews[7] = &m_ParamNameView8;
	m_ParamNameViews[8] = &m_ParamNameView9;
	m_ParamNameViews[9] = &m_ParamNameView10;

	m_ParamValueViews[0] = &m_ParamValueView1;
	m_ParamValueViews[1] = &m_ParamValueView2;
	m_ParamValueViews[2] = &m_ParamValueView3;
	m_ParamValueViews[3] = &m_ParamValueView4;
	m_ParamValueViews[4] = &m_ParamValueView5;
	m_ParamValueViews[5] = &m_ParamValueView6;
	m_ParamValueViews[6] = &m_ParamValueView7;
	m_ParamValueViews[7] = &m_ParamValueView8;
	m_ParamValueViews[8] = &m_ParamValueView9;
	m_ParamValueViews[9] = &m_ParamValueView10;

	m_ParamSliderViews[0] = &m_ParamSliderView1;
	m_ParamSliderViews[1] = &m_ParamSliderView2;
	m_ParamSliderViews[2] = &m_ParamSliderView3;
	m_ParamSliderViews[3] = &m_ParamSliderView4;
	m_ParamSliderViews[4] = &m_ParamSliderView5;
	m_ParamSliderViews[5] = &m_ParamSliderView6;
	m_ParamSliderViews[6] = &m_ParamSliderView7;
	m_ParamSliderViews[7] = &m_ParamSliderView8;
	m_ParamSliderViews[8] = &m_ParamSliderView9;
	m_ParamSliderViews[9] = &m_ParamSliderView10;

	ShowAllParamViews(false);
	return TRUE;
}
BEGIN_MESSAGE_MAP(CTMGRealTimeAudioDlg, CDialogEx)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_BUTTON_ENTERROOM, &CTMGRealTimeAudioDlg::OnBnClickedButtonEnterroom)
	ON_BN_CLICKED(IDC_BUTTON_EXITROOM, &CTMGRealTimeAudioDlg::OnBnClickedButtonExitroom)
	ON_BN_CLICKED(IDC_CHECK_ENABLEMIC, &CTMGRealTimeAudioDlg::OnBnClickedCheckEnablemic)
	ON_BN_CLICKED(IDC_CHECK_ENABLESPEAKER, &CTMGRealTimeAudioDlg::OnBnClickedCheckEnablespeaker)
	ON_CBN_SELCHANGE(IDC_COMBO_ROOMTYPE, &CTMGRealTimeAudioDlg::OnCbnSelchangeComboRoomtype)
	ON_CBN_SELCHANGE(IDC_COMBO_VOICETYPE, &CTMGRealTimeAudioDlg::OnCbnSelchangeComboVoicetype)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_SLIDER_PARAM1, &CTMGRealTimeAudioDlg::OnNMCustomdrawSliderParam1)
	ON_BN_CLICKED(IDC_BUTTON_RESETPARAM, &CTMGRealTimeAudioDlg::OnBnClickedButtonResetparam)
	ON_BN_CLICKED(IDC_CHECK_ENABLELOOPBACK, &CTMGRealTimeAudioDlg::OnBnClickedCheckEnableloopback)
END_MESSAGE_MAP()



// Handler EnterRoomClick
void CTMGRealTimeAudioDlg::OnBnClickedButtonEnterroom()
{
	CString cstrRoomId;
	m_RoomId.GetWindowTextW(cstrRoomId);
	std::string utf8RoomId = CUtils::WCharToUTF8(cstrRoomId).GetString();

	unsigned int authBufferLen = 4096;
	char *pAuthBuffer = new char[authBufferLen];
	memset(pAuthBuffer, 0, authBufferLen);
	m_pHostDlg->CreateAuthBuffer(utf8RoomId.c_str(), pAuthBuffer, authBufferLen);

	
	// Step 6/12 enter room, then you need handle ITMG_MAIN_EVENT_TYPE_ENTER_ROOM,ITMG_MAIN_EVNET_TYPE_USER_UPDATE
	//  and ITMG_MAIN_EVENT_TYPE_ROOM_DISCONNECT event in OnEvent function in CTMGSDK_For_Audio_ApiExampleDlg class
	// https://cloud.tencent.com/document/product/607/15232#.E5.8A.A0.E5.85.A5.E6.88.BF.E9.97.B4
	ITMGContext* pTmgContext = ITMGContextGetInstance();
	pTmgContext->EnterRoom(utf8RoomId.c_str(), (ITMG_ROOM_TYPE)(m_ComBoxRoomType.GetCurSel() + 1), (const char*)pAuthBuffer, authBufferLen);
	delete[] pAuthBuffer;
	m_EnterRoomBtn.EnableWindow(false);
}

void CTMGRealTimeAudioDlg::OnBnClickedCheckEnablemic()
{
	// Step 9/12 enable to capture audio and send to server
	// https://cloud.tencent.com/document/product/607/15232#.E5.BC.80.E5.90.AF.E6.88.96.E5.85.B3.E9.97.AD.E9.BA.A6.E5.85.8B.E9.A3.8E
	ITMGContextGetInstance()->GetAudioCtrl()->EnableMic(m_EnableMicCheckBox.GetCheck() == TRUE);
}


void CTMGRealTimeAudioDlg::OnBnClickedCheckEnablespeaker()
{
	// Step 10/12 enable to capture audio and send to server
	// https://cloud.tencent.com/document/product/607/15232#.E5.BC.80.E5.90.AF.E6.88.96.E5.85.B3.E9.97.AD.E6.89.AC.E5.A3.B0.E5.99.A8
	ITMGContextGetInstance()->GetAudioCtrl()->EnableSpeaker(m_EnableSpeakerCheckBox.GetCheck() == TRUE);
}

// Hanlde ExitRoomClick
void CTMGRealTimeAudioDlg::OnBnClickedButtonExitroom()
{
	// Step 11/12 exit room
	// https://cloud.tencent.com/document/product/607/15232#.E9.80.80.E5.87.BA.E6.88.BF.E9.97.B4
	ITMGContextGetInstance()->ExitRoom();
	m_EnterRoomBtn.EnableWindow(true);
}

void CTMGRealTimeAudioDlg::OnCbnSelchangeComboRoomtype()
{
	// TODO: Add your control notification handler code here
	ITMGContextGetInstance()->GetRoom()->ChangeRoomType((ITMG_ROOM_TYPE)(m_ComBoxRoomType.GetCurSel() + 1));
}

bool compFn(const CVoiceParam &a, const CVoiceParam &b) {
	return a.m_strParamName < b.m_strParamName;
}

void CTMGRealTimeAudioDlg::OnCbnSelchangeComboVoicetype() {
	// TODO: Add your control notification handler code here
	int index = m_ComBoxVoiceType.GetCurSel();
	std::string voiceName = m_vVoiceList[index];
	if (index == 0) {
		voiceName = "";
	}
	int nRet = ITMGContextGetInstance()->GetAudioEffectCtrl()->SetVoiceChangerName(voiceName.c_str());
	m_vParamList.clear();
	ShowAllParamViews(false);
	if (nRet == nsgme::av::AV_OK && index != 0) {
		const char* voiceParams = ITMGContextGetInstance()->GetAudioEffectCtrl()->GetVoiceChangerParams();
		nsgme::av::Json::Reader reader;
		nsgme::av::Json::Value root;
		bool parseRet = reader.parse(voiceParams, root);
		if (!parseRet) {
			return;
		}
		for (int i = 0; i < root.size(); i++) {
			std::vector<std::string> memberNames = root[i].getMemberNames();
			if (memberNames.size() > 0) {
				std::string paramName = memberNames[0];
				//if (paramName.find("_") == 0) {
				//	continue;
				//}
				CVoiceParam param = CVoiceParam(paramName, root[i][paramName]["value"].asFloat(), 
					root[i][paramName]["minvalue"].asFloat(), 
					root[i][paramName]["maxvalue"].asFloat(),
					root[i][paramName]["default"].asFloat());
				m_vParamList.push_back(param);
			}
		}
		std::sort(m_vParamList.begin(), m_vParamList.end(), compFn);
		RefreshParamViews();	
	}
}

void CTMGRealTimeAudioDlg::RefreshParamViews() {
	int paramsSize = m_vParamList.size();
	int count = paramsSize > COUNT_DEFAULT_PARAM_VIEW ? COUNT_DEFAULT_PARAM_VIEW : paramsSize;
	// update Value
	for (size_t i = 0; i < count; i++) {
		m_ParamNameViews[i]->SetWindowTextW(CUtils::UTF8ToWChar(m_vParamList[i].m_strParamName).c_str());
		m_ParamNameViews[i]->ShowWindow(true);

		m_ParamValueViews[i]->SetWindowTextW(CUtils::UTF8ToWChar(CUtils::ToString(m_vParamList[i].m_fValue)).c_str());
		m_ParamValueViews[i]->ShowWindow(true);

		m_ParamSliderViews[i]->SetRange(m_vParamList[i].m_fMinValue * 1000, m_vParamList[i].m_fMaxValue * 1000, true);
		m_ParamSliderViews[i]->SetPos(m_vParamList[i].m_fValue * 1000);
		m_ParamSliderViews[i]->ShowWindow(true);
	}
	if (count > 0) {
		m_ResetParamBtn.ShowWindow(true);
	}
}

void CTMGRealTimeAudioDlg::OnNMCustomdrawSliderParam1(NMHDR *pNMHDR, LRESULT *pResult) {
	LPNMCUSTOMDRAW pNMCD = reinterpret_cast<LPNMCUSTOMDRAW>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;
}

void CTMGRealTimeAudioDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) {
	for (size_t i = 0; i < COUNT_DEFAULT_PARAM_VIEW; i++) {
		if ((CWnd*)pScrollBar == (CWnd*)m_ParamSliderViews[i]) {
			int pos = m_ParamSliderViews[i]->GetPos();
			m_vParamList[i].m_fValue = pos*1.0f / 1000;
			m_ParamValueViews[i]->SetWindowTextW(CUtils::UTF8ToWChar(CUtils::ToString(m_vParamList[i].m_fValue)).c_str());
			int nRet = ITMGContextGetInstance()->GetAudioEffectCtrl()->SetVoiceChangerParamValue(m_vParamList[i].m_strParamName.c_str(),
																								 m_vParamList[i].m_fValue);
		}
	}
	
}

void CTMGRealTimeAudioDlg::OnBnClickedButtonResetparam() {
	// TODO: Add your control notification handler code here
	for (size_t i = 0; i < m_vParamList.size(); i++) {
		m_vParamList[i].m_fValue = m_vParamList[i].m_fDefaultValue;
		ITMGContextGetInstance()->GetAudioEffectCtrl()->SetVoiceChangerParamValue(m_vParamList[i].m_strParamName.c_str(), m_vParamList[i].m_fValue);
	}
	RefreshParamViews();
}

void CTMGRealTimeAudioDlg::OnBnClickedCheckEnableloopback() {
	// TODO: Add your control notification handler code here
	ITMGContextGetInstance()->GetAudioCtrl()->EnableLoopBack(m_EnableLoopbackCheckBox.GetCheck());
}
